#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal_in;
layout(location = 2) in vec2 texture_coord_in;

out VertexData {
	vec3 position_world;
	vec3 Normal;
	vec2 texture_coord;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform float animationTime;


void main() {
	vec3 animatedPos = position;
	float translationAmount = sin(animationTime) * 0.5f;
	float rotationAmount = sin(animationTime) * 0.5f;
	
	//animatedPos += torusCenter;
	animatedPos.y += translationAmount;

	mat4 rotationMatrix = mat4(
        vec4(cos(rotationAmount), 0.0, sin(rotationAmount), 0.0),
        vec4(0.0, 1.0, 0.0, 0.0),
        vec4(-sin(rotationAmount), 0.0, cos(rotationAmount), 0.0),
        vec4(0.0, 0.0, 0.0, 1.0)
    );
    vec4 rotatedPos = rotationMatrix * vec4(animatedPos, 1.0);


	vec4 animatedPositionWorld = modelMatrix * rotatedPos;

	// gl_Position is a built-in variable
	gl_Position = viewProjMatrix * animatedPositionWorld;
	vert.position_world = vec3(animatedPositionWorld);
	vert.Normal = normal_in;
	vert.texture_coord = texture_coord_in;
}