#version 330 core
layout (location = 0) in vec3 aPos;
layout (location = 1) in vec3 aNormal;

out vec3 Normal;
out vec3 Position;

uniform mat4 model;
uniform mat4 viewProj;

uniform float animationTime;

uniform float wobbleFrequency;
uniform float wobbleAmplitude;


void main()
{
    vec3 animatedPos = aPos;
    float wobbleOffset = sin(aPos.x * wobbleFrequency + animationTime) * wobbleAmplitude;
    animatedPos += vec3(0.0f, wobbleOffset, 0.0f);

    vec4 animatedPositionWorld = model * vec4(animatedPos, 1.0);

    // Calculate transformed normal
    mat3 normalMatrix = transpose(inverse(mat3(model)));
    vec3 transformedNormal = normalize(normalMatrix * aNormal);

    Normal = transformedNormal;
    Position = vec3(animatedPositionWorld);
    gl_Position = viewProj * animatedPositionWorld;
}
